@@ -8,7 +8,7 @@ use Ref::Util qw( is_coderef is_arrayref );
 use if $^O eq 'MSWin32', 'Win32';
 
 # ABSTRACT: Basic implementation of the XDG base directory specification
-our $VERSION = '0.09_01'; # TRIAL VERSION
+our $VERSION = '0.10_01'; # TRIAL VERSION
 $VERSION = eval $VERSION; ## no critic (BuiltinFunctions::ProhibitStringyEval)
 
 
@@ -67,6 +67,10 @@ sub new {
       Carp::croak("Unknown path class: $path_class");
     }
 
+    my $strict = delete $args{strict};
+    Carp::croak("XDG base directory specification cannot strictly implemented on Windows")
+      if $^O eq 'MSWin32' && $strict;
+
     Carp::croak("unknown arguments: @{[ sort keys %args ]}") if %args;
 
     my $self = bless {
@@ -74,19 +78,25 @@ sub new {
         api        => $api,
         file_class => $file_class,
         dir_class  => $dir_class,
+        strict     => $strict,
+        runtime    => $ENV{XDG_RUNTIME_DIR},
     }, $class;
 
     if($^O eq 'MSWin32') {
         my $local = Win32::GetFolderPath(Win32::CSIDL_LOCAL_APPDATA(), 1);
+        $self->{home}        = $local;
         $self->{data}        = $ENV{XDG_DATA_HOME}   || "$local\\.local\\share\\";
         $self->{cache}       = $ENV{XDG_CACHE_HOME}  || "$local\\.cache\\";
         $self->{config}      = $ENV{XDG_CONFIG_HOME} || "$local\\.config\\";
+        $self->{state}       = $ENV{XDG_STATE_HOME}  || "$local\\.state\\";
         $self->{data_dirs}   = $ENV{XDG_DATA_DIRS}   || '';
         $self->{config_dirs} = $ENV{XDG_CONFIG_DIRS} || '';
     } else {
         my $home = $ENV{HOME} || [getpwuid($>)]->[7];
+        $self->{home}        = $home;
         $self->{data}        = $ENV{XDG_DATA_HOME}   || "$home/.local/share/";
         $self->{cache}       = $ENV{XDG_CACHE_HOME}  || "$home/.cache/";
+        $self->{state}       = $ENV{XDG_STATE_HOME}  || "$home/.state/";
         $self->{config}      = $ENV{XDG_CONFIG_HOME} || "$home/.config/";
         $self->{data_dirs}   = $ENV{XDG_DATA_DIRS}   || '/usr/local/share:/usr/share';
         $self->{config_dirs} = $ENV{XDG_CONFIG_DIRS} || '/etc/xdg';
@@ -150,6 +160,54 @@ sub cache_home {
 }
 
 
+sub state_home {
+  my $self = shift;
+  return $self->_dir($self->{state}, $self->{name});
+}
+
+
+sub runtime_home
+{
+  my($self) = @_;
+  my $base = $self->_runtime_dir;
+  defined $base ? $self->_dir($base, $self->{name}) : undef;
+}
+
+sub _runtime_dir
+{
+  my($self) = @_;
+  if(defined $self->{runtime})
+  {
+    return $self->{runtime};
+  }
+
+  # the spec says only to look for the environment variable
+  return undef if $self->{strict};
+
+  my @maybe;
+
+  if($^O eq 'linux')
+  {
+    push @maybe, "/run/user/$<";
+  }
+
+  foreach my $maybe (@maybe)
+  {
+    # if we are going rogue and trying to find the runtime dir
+    # on our own, then we hould at least check that the directory
+    # fufills the requirements of the spec: directory, owned by
+    # us, with permission of 0700.
+    next unless -d $maybe;
+    next unless -o $maybe;
+    my $perm = [stat $maybe]->[2] & oct('0777');
+    next unless $perm == oct('0700');
+    return $maybe;
+  }
+
+  return undef;
+}
+
+
 sub data_dirs {
     return shift->_dirs('data');
 }
@@ -172,6 +230,15 @@ sub config_dirs_list {
 }
 
 
+sub exe_dir
+{
+  my($self) = @_;
+  -d "@{[ $self->{home} ]}/.local/bin"
+  ? $self->_dir($self->{home}, '.local', 'bin')
+  : undef;
+}
+
+
 sub lookup_data_file {
     my ($self, @subpath) = @_;
     unshift @subpath, $self->{name} if $self->{api} >= 1;
@@ -200,7 +267,7 @@ File::XDG - Basic implementation of the XDG base directory specification
 
 =head1 VERSION
 
-version 0.09_01
+version 0.10_01
 
 =head1 SYNOPSIS
 
@@ -245,6 +312,8 @@ Takes the following named arguments:
 
 =item api
 
+[version 0.09]
+
 The API version to use.
 
 =over 4
@@ -267,9 +336,11 @@ Name of the application for which File::XDG is being used.
 
 =item path_class
 
+[version 0.09]
+
 The path class to return
 
-=over 4
+=over
 
 =item L<File::Spec>
 
@@ -316,6 +387,32 @@ between files and directories.
 
 =back
 
+=item strict
+
+[version 0.10]
+
+More strictly follow the XDG base directory specification.  In particular
+
+=over 4
+
+=item
+
+On Windows a an exception will be thrown when creating the L<File::XDG>
+object because the spec cannot correctly be implemented.
+
+Historically this module has made some useful assumptions like using
+C<;> instead of C<:> for the path separator character.  This breaks the
+spec.
+
+=item
+
+On some systems, this module will look in system specific locations for
+the L</runtime_home>.  This is useful, but technically violates the spec,
+so under strict mode the L</runtime_home> method will only return a path
+if one can be found via the spec.
+
+=back
+
 =back
 
 =head1 METHODS
@@ -338,6 +435,35 @@ Returns the user-specific configuration directory for the application as a path
 
 Returns the user-specific cache directory for the application as a path class object.
 
+=head2 state_home
+
+ my $path = $xdg->state_home;
+
+Returns the user-specific state directory for the application as a path class object.
+
+=head2 runtime_home
+
+[version 0.10]
+
+ my $dir = $xdg->runtime_home;
+
+Returns the directory for user-specific non-essential runtime files and other file objects
+(such as sockets, named pipes, etc) for the application.
+
+This is not always provided, if not available, this method will return C<undef>.
+
+Under strict mode, this method will only rely on the C<XDG_RUNTIME_DIR> to find this directory.
+Under non-strict mode, system specific methods may be used, if the environment variable is not
+set:
+
+=over 4
+
+=item Linux systemd
+
+The path C</run/user/UID> will be used, if it exists, and fulfills the requirements of the spec.
+
+=back
+
 =head2 data_dirs
 
  my $dirs = $xdg->data_dirs;
@@ -374,6 +500,16 @@ For portability L</config_dirs_list> is preferred.
 
 Returns the system config directories as a list of path class objects.
 
+=head2 exe_dir
+
+[version 0.10]
+
+ my $exe = $xdg->exe_dir;
+
+Returns the user-specific executable files directory C<$HOME/.local/bin>, if it exists.  If it
+does not exist then C<undef> will be returned.  This directory I<should> be added to the C<PATH>
+according to the spec.
+
 =head2 lookup_data_file
 
  # api = 0
